Learn R Programming

soilhypfit (version 0.1-8)

sim_wrc_hcc: Simulated Soil Water Retention and Hydraulic Conductivity Data

Description

The data give simulated values of water content and hydraulic conductivity at given capillary pressure head for 3 soil samples.

Usage

data(sim_wrc_hcc)

Arguments

Format

A data frame with 28 observations on the following 4 variables.

id

a factor with levels 1, 2, 3 coding the soil samples.

head

a numeric vector with values of capillary pressure head (unit \(\mathrm{m}\)).

wc

a numeric vector with values of simulated volumetric water content (unit -)

hc

a numeric vector with values of simulated hydraulic conductivity (unit \(\mathrm{m} \, \mathrm{d}^{-1}\)).

Details

The values of wc and hc were simulated by the model of Van Genuchten Mualem (Van Genuchten, 1980, see wc_model and hc_model) with the following parameters:

sample id\(\theta_r\)\(\theta_s\)\(K_0\) [\(\mathrm{m} \, \mathrm{d}^{-1}\))]\(\alpha\) [\(\mathrm{m}^{-1}\)]\(n\)\(\tau\)
10.050.450.1220.5
20.10.551.51.50.5
30.050.4521.41.30.5

Normally distributed errors were added to the model values (wc: sd: 0.05; log(hc): sd 0.5).

References

Van Genuchten, M. Th. (1980) A closed-form equation for predicting the hydraulic conductivity of unsaturated soils. Soil Science Society of America Journal, 44, 892--898, tools:::Rd_expr_doi("10.2136/sssaj1980.03615995004400050002x").

Examples

Run this code
data(sim_wrc_hcc)

if(!requireNamespace("lattice", quietly = TRUE)){
  install.packages("lattice", repos = "https://cloud.r-project.org/")
  requireNamespace("lattice", quietly = TRUE)
}

lattice::xyplot(wc ~ head|id, type = "l", sim_wrc_hcc, 
  scales = list( x = list(log=TRUE)))
lattice::xyplot(hc ~ head|id, type = "l", sim_wrc_hcc, 
  scales = list( log = TRUE))

Run the code above in your browser using DataLab