| swissforestsoils | R Documentation |
The data give basic physical properties, water content and hydraulic conductivity measurements (at given capillary pressure head) for 128 soil layers (horizons) measured at 23 forest sites in Switzerland.
data(swissforestsoils)
A data frame with 1373 observations on the following 21 variables.
profile_ida factor with short labels for the 23 sites.
profilea factor with with long labels for the 23 sites.
longitudea numeric vector with the latitude of the site in degree.
latitudea numeric vector with the latitude of the site in degree.
layer_ida factor with labels for the 128 soil layer.
layer_ub, layer_lbnumeric vectors with the upper
and lower depth (unit cm) of the soil layer for the measurements of the
basic physical properties (particle_density, ..., ksat).
particle_densitya numeric vector with the density of the
solid soil material (unit \mathrm{g} \,\mathrm{cm^{-3}}).
bulk_densitya numeric vector with soil (bulk) density
(unit \mathrm{g} \,\mathrm{cm^{-3}}).
porositya numeric vector with the soil porosity (unit volume percentage).
claya numeric vector with the clay content (unit mass percentage).
silta numeric vector with the silt content (unit mass percentage).
sanda numeric vector with the sand content (unit mass percentage).
ksata numeric vector with the saturated hydraulic
conductivity (unit \mathrm{m} \,\mathrm{d^{-1}}).
heada numeric vector with capillary pressure head at
which theta (water retention curve) and/or ku (hydraulic
conductivity function) were measured (unit m).
layer_ub_theta, layer_lb_thetanumeric vectors with the upper and lower depth (unit cm) of the soil layer for which the water retention curve was measured.
thetaa numeric vector with volumetric water content measurements (dimensionless) of the water retention curve.
layer_ub_ku, layer_lb_kua numeric vector with the upper and lower depth (unit cm) of the soil layer for which the water retention curve was measured.
kua numeric vector with hydraulic conductivity
measurements (unit \mathrm{m} \,\mathrm{d^{-1}}) of
the hydraulic conductivity function.
clay, silt and sand refer to soil particles with
diameter less than 2, between 2 and 50 and larger than 50 \mum.
Richard, F. & Lüscher, P. 1978 – 1987. Physikalische Eigenschaften von Böden der Schweiz. Lokalformen Bände 1 – 4. Eidgenössische Anstalt für das forstliche Versuchswesen, Birmensdorf.
# use of \donttest{} because execution time exceeds 5 seconds
# estimate parameters using all samples (samples with water retention,
# hydraulic conductivity, or with both type of measurements)
data(swissforestsoils)
# define number of cores for parallel computations
if(interactive()) ncpu <- parallel::detectCores() - 1L else ncpu <- 1L
# unconstrained estimation (global optimisation algorithm NLOPT_GN_MLSL)
r_uglob <- fit_wrc_hcc(
wrc_formula = theta ~ head | layer_id,
hcc_formula = ku ~ head | layer_id,
data = swissforestsoils,
control = control_fit_wrc_hcc(
settings = "uglobal", pcmp = control_pcmp(ncores = ncpu)))
summary(r_uglob)
coef(r_uglob)
Add the following code to your website.
For more information on customizing the embed code, read Embedding Snippets.