A data frame with 1373 observations on the following 21 variables.
profile_id
a factor with short labels for the 23 sites.
profile
a factor with with long labels for the 23 sites.
longitude
a numeric vector with the latitude of the site
in degree.
latitude
a numeric vector with the latitude of the site
in degree.
layer_id
a factor with labels for the 128 soil layer.
layer_ub, layer_lb
numeric vectors with the upper
and lower depth (unit cm) of the soil layer for the measurements of the
basic physical properties (particle_density, ..., ksat).
particle_density
a numeric vector with the density of the
solid soil material (unit \(\mathrm{g} \,\mathrm{cm^{-3}}\)).
bulk_density
a numeric vector with soil (bulk) density
(unit \(\mathrm{g} \,\mathrm{cm^{-3}}\)).
porosity
a numeric vector with the soil porosity (unit
volume percentage).
clay
a numeric vector with the clay content (unit mass
percentage).
silt
a numeric vector with the silt content (unit mass
percentage).
sand
a numeric vector with the sand content (unit mass
percentage).
ksat
a numeric vector with the saturated hydraulic
conductivity (unit \(\mathrm{m} \,\mathrm{d^{-1}}\)).
head
a numeric vector with capillary pressure head at
which theta (water retention curve) and/or ku (hydraulic
conductivity function) were measured (unit m).
layer_ub_theta, layer_lb_theta
numeric vectors
with the upper and lower depth (unit cm) of the soil layer for which
the water retention curve was measured.
theta
a numeric vector with volumetric water content
measurements (dimensionless) of the water retention curve.
layer_ub_ku, layer_lb_ku
a numeric vector with the
upper and lower depth (unit cm) of the soil layer for which the water
retention curve was measured.
ku
a numeric vector with hydraulic conductivity
measurements (unit \(\mathrm{m} \,\mathrm{d^{-1}}\)) of
the hydraulic conductivity function.