This page documents the functions convergence_message,
extract_error_messages, select_failed_fits and
check_param_boundf.
convergence_message(x, sce = FALSE)extract_error_messages(object, start = 1, stop = 80, prt = TRUE)
select_failed_fits(object)
check_param_boundf(y, compare_thetar_thetas = TRUE)
The function convergence_message and extract_error_messages
return invisibly the convergence code or the error messages.
an integer scalar issued by the optimisers of
nloptr or SCEoptim on
(non-)convergence
a logical scalar to select the optimiser
nloptr (FALSE, default) or
SCEoptim (TRUE).
an object of class fit_wrc_hcc, see
fit_wrc_hcc.
a logical scalar controlling whether the error messages should be printed.
integer scalar with the first and last character to print.
a named list of numeric vectors of length 2 that define the
allowed lower and upper bounds (box constraints) for the parameters of
the models, see argument param_bound of
control_fit_wrc_hcc.
logical scalar to control cross-comparison
of valid ranges of parameters thetar and thetas.
Andreas Papritz papritz@retired.ethz.ch.
The function convergence_message prints a message that explains
the convergence codes, for nloptr, see
NLopt
return values. The function extract_error_messages extract the
error messages of estimations that failed and optionally prints sub-strings
of them.
The function select_failed_fits returns the
ids of the soil samples for which parameter estimation failed.
The function check_param_boundf checks the validity and consistecy
of bounds of box constraints of model parameters.
Duan, Q., Sorooshian, S., and Gupta, V. K. (1994) Optimal use of the SCE-UA global optimisation method for calibrating watershed models, Journal of Hydrology 158, 265--284, tools:::Rd_expr_doi("10.1016/0022-1694(94)90057-4").
Johnson, S.G. The NLopt nonlinear-optimisation package. https://github.com/stevengj/nlopt.
soilhypfitIntro for a description of the models and a brief
summary of the parameter estimation approach;
fit_wrc_hcc for (constrained) estimation of parameters of
models for soil water retention and hydraulic conductivity data;
control_fit_wrc_hcc for options to control
fit_wrc_hcc;
soilhypfitmethods for common S3 methods for class
fit_wrc_hcc;
vcov for computing (co-)variances of the estimated
nonlinear parameters;
prfloglik_sample for profile loglikelihood
computations;
wc_model and hc_model for currently
implemented models for soil water retention curves and hydraulic
conductivity functions;
evaporative-length for physically constraining parameter
estimates of soil hydraulic material functions.
convergence_message(3)
convergence_message(2, sce = TRUE)
Run the code above in your browser using DataLab