| sim_wrc_hcc | R Documentation |
The data give simulated values of water content and hydraulic conductivity at given capillary pressure head for 3 soil samples.
data(sim_wrc_hcc)
A data frame with 28 observations on the following 4 variables.
ida factor with levels 1, 2,
3 coding the soil samples.
heada numeric vector with values of capillary pressure
head (unit \mathrm{m}).
wca numeric vector with values of simulated volumetric water content (unit -)
hca numeric vector with values of simulated hydraulic
conductivity (unit \mathrm{m} \, \mathrm{d}^{-1}).
The values of wc and hc were simulated by the model of
Van Genuchten Mualem (Van Genuchten, 1980, see
wc_model and hc_model) with the following
parameters:
| sample id | \theta_r | \theta_s | K_0 [\mathrm{m} \, \mathrm{d}^{-1})]
| \alpha [\mathrm{m}^{-1}] | n | \tau |
| 1 | 0.05 | 0.45 | 0.1 | 2 | 2 | 0.5 |
| 2 | 0.1 | 0.5 | 5 | 1.5 | 1.5 | 0.5 |
| 3 | 0.05 | 0.45 | 2 | 1.4 | 1.3 | 0.5 |
Normally distributed errors were added to the model values (wc:
sd: 0.05; log(hc): sd 0.5).
Van Genuchten, M. Th. (1980) A closed-form equation for predicting the hydraulic conductivity of unsaturated soils. Soil Science Society of America Journal, 44, 892–898, \Sexpr[results=rd]{tools:::Rd_expr_doi("10.2136/sssaj1980.03615995004400050002x")}.
data(sim_wrc_hcc)
if(!requireNamespace("lattice", quietly = TRUE)){
install.packages("lattice", repos = "https://cloud.r-project.org/")
requireNamespace("lattice", quietly = TRUE)
}
lattice::xyplot(wc ~ head|id, type = "l", sim_wrc_hcc,
scales = list( x = list(log=TRUE)))
lattice::xyplot(hc ~ head|id, type = "l", sim_wrc_hcc,
scales = list( log = TRUE))
Add the following code to your website.
For more information on customizing the embed code, read Embedding Snippets.